/***********************************************************************/
/*                                                                     */
/*                              Pio (C++)                              */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*               EPI Pomdapi, INRIA Paris - Rocquencourt               */
/*                                                                     */
/*  Copyright 2005-2014 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                       */
/*                                                                     */
/***********************************************************************/

/* $Id: receive_typed_value.cpp,v 1.9 2014/01/10 19:22:09 fclement Exp $ */

#include "receive_typed_value.hpp"

namespace io{
  
   //defined in single_types.cpp:
  void set_reception_binary();
  void set_reception_ascii();

  
  //sub-routines defined in receive.cpp:
  string receive_value_same_layout(istream& ib,unsigned int& m,unsigned int& n,double*& M);
  void r_begin(istream& ib,string f1);
  void r_end(istream& ib,string f1);
  template<class typeName> void receive_value(istream& ib,vector<typeName>& result );
  template<class typeName> void receive_value(istream& ib,typeName& t);
  void receive_value(istream& ib,unsigned int& m,unsigned int& n,double** & M,string layout="row");
 
  //|--------------------------------|
  //|to receive generic typed_values:|
  //|--------------------------------|
  
  //function to read the second line representing a Typed_calue in a communication.
  //this second line is returned as a string.
  string receive_type_for_typed_value(istream& ib){
    char t[20];
    ib.getline(t,20,'\n');
    string temp(t);
    return temp;
  }
  
  //We suppose that the first two lines of the part of a communication representing a Typed value have been read,  so we know the complete type of the typed value, represented by a string 's' : we can call one of the four following funtion.
  Lexem receive_scalar_typed_value(istream& ib,string s){
    if(s=="%i"){
      int i;receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="%S"){
      string i;
      receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="%B"){
      boolean i;
      receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="(%B, %B)"){
      boolean_couple i;
      receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="(%B, %B, %B)"){
      boolean_triple i;
      receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="(%i, %i)"){
      int_couple i;
      receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="(%i, %i, %i)"){
      int_triple i;
      receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="%bf"){
      set_reception_binary();
      double i;
      receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="(%bf, %bf)"){
      set_reception_binary();
      float_couple i;
      receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="(%bf, %bf, %bf)"){
      set_reception_binary();
      float_triple i;
      receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="%f"){
      set_reception_ascii();
      double i;receive_value(ib,i);Lexem p(i);return p;
    }
    else if(s=="(%f, %f)"){
      set_reception_ascii();
      float_couple i;receive_value(ib,i);Lexem p(i);
      return p;
    }
    else if(s=="(%f, %f, %f)"){
      set_reception_ascii();
      float_triple i;receive_value(ib,i);Lexem p(i);
      return p;
    }
    else{
      cerr<<"wrong format "<<s<<" for Lexem"<<endl;
      exit(1);
    }
  }
  
  Vector_type receive_vector_typed_value(istream& ib,string s){
    if(s=="[1%i1]"){
      vector<int> i;
      receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1%S1]"){
      vector<string> i;
      receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1(%i, %i)1]"){
      vector<int_couple> i;
      receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1(%i, %i, %i)1]"){
      vector<int_triple> i;
      receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1%B1]"){
      vector<boolean> i;
      receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1(%B, %B)1]"){
      vector<boolean_couple> i;
      receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1(%B, %B, %B)1]"){
      vector<boolean_triple> i;
      receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1%bf1]"){
      set_reception_binary();
      vector<double> i;receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1(%bf, %bf)1]"){
      set_reception_binary();
      vector<float_couple> i;receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1(%bf, %bf, %bf)1]"){
      set_reception_binary();
      vector<float_triple> i;receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1%f1]"){
      set_reception_ascii();
      vector<double> i;receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1(%f, %f)1]"){
      set_reception_ascii();
      vector<float_couple> i;receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else if(s=="[1(%f, %f, %f)1]"){
      set_reception_ascii();
      vector<float_triple> i;receive_value(ib,i);Vector_type p(i);
      return p;
    }
    else{
      cerr<<"wrong format "<<s<<" for Vector_type"<<endl;
      exit(1);
    }
  }
  
  Matrix_type1 receive_matrix1_typed_value(istream& ib,string s){
    if(s=="[2%bf2]"){
      set_reception_binary();
      unsigned int m,n;
      double*  M;
      string s=receive_value_same_layout(ib,m,n,M);
      char c=((s=="row")?'C':'F');
      Matrix_type1 p(m,n,M,c); 
      return p;
    }
    else if(s=="[2%f2]"){
      set_reception_ascii();
      unsigned int m,n;
      double*  M;
      string s=receive_value_same_layout(ib,m,n,M);
      char c=((s=="row")?'C':'F');
      Matrix_type1 p(m,n,M,c); 
      return p;
    }
    else{
      cerr<<"wrong format for Matrix_type1"<<endl;
      exit(1);
    }
  }
  
  Matrix_type2 receive_matrix2_typed_value(istream& ib,string s){
    if(s=="[2%bf2]"){
      set_reception_binary();
      unsigned int m,n;
      double**  M;
      receive_value(ib,m,n,M);
      Matrix_type2 p(m,n,M);
      return p;
    }
    else if(s=="[2%f2]"){
      set_reception_ascii();
      unsigned int m,n;
      double**  M;
      receive_value(ib,m,n,M);
      Matrix_type2 p(m,n,M);
      return p;
    }
    else{
      cerr<<"wrong format "<<s<<" for Matrix_type2"<<endl;
      exit(1);
    }
  }
  
  
  //Once the first two lines of a communication read ...
  Typed_value receive(istream& ib){
    string f1 = "receive Typed_value";
    r_begin(ib,f1);
    string fmt=receive_type_for_typed_value(ib);
    if(fmt.data()[0]=='%' || fmt.data()[0]=='('){
      Typed_value p(receive_scalar_typed_value(ib,fmt));
      r_end(ib,f1);
      return p;
    }
    else if(fmt.data()[0]=='[' && fmt.data()[1]=='1'){
      Typed_value p(receive_vector_typed_value(ib,fmt));
      r_end(ib,f1);
      return p;
    }
    else if(fmt.data()[0]=='[' && fmt.data()[1]=='2'){
      Typed_value p(receive_matrix1_typed_value(ib,fmt));
      r_end(ib,f1);
      return p;
    }
    else{
      cerr<<"wrong format "<<fmt<<" for Typed_value"<<endl;
      exit(1);
    }
  }

  Typed_value receive_typed_value(istream& ib){
    return receive(ib);
  }
  
}
