/***********************************************************************/
/*                                                                     */
/*                               Pio (C)                               */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*               EPI Pomdapi, INRIA Paris - Rocquencourt               */
/*                                                                     */
/*  Copyright 2011-2014 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Bruno Verlyck <Bruno.Verlyck@inria.fr>                             */
/*                                                                     */
/***********************************************************************/

#include <assert.h>
/* only use assert (0) */
#undef NDEBUG

#include <setjmp.h>
#include <stdbool.h>
#include <stdint.h>
#include <stddef.h>

typedef intptr_t natint;
typedef int32_t int32;
typedef int64_t int64;

#define __max(a,b) (((a) > (b)) ? (a) : (b))
#define __min(a,b) (((a) < (b)) ? (a) : (b))

/* raise exn */
jmp_buf *fail_label;
const char *failure_msg;
void failwith (const char *msg) __attribute__ ((noreturn));
#define error(fun_name, msg) failwith (fun_name ": " msg)

char failbuf[512];			/* to snprintf the failwith msg */
#define failwithf(format, ...) failwith((snprintf (failbuf, sizeof (failbuf), format, __VA_ARGS__), failbuf))

/* unconditional abort */
void invalid_argument (const char *msg) __attribute__ ((noreturn));

/* This is done for immutable strings. */
struct string {
  size_t ssize;
  const char *str;
};
int string_cmp (struct string *s1, struct string *s2);
#define string_equal(s1, s2) (string_cmp (s1, s2) == 0)
int sstring_cmp (struct string s1, struct string s2);
#define sstring_equal(s1, s2) (sstring_cmp (s1, s2) == 0)

bool binary_mode, report_mode, debug_mode;
#define ascii_mode (!binary_mode)
