(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: zonation.ml,v 1.22 2014-02-26 23:33:54 fclement Exp $ *)

module Nat = Unatural;;

module Set = Umatrix_nonempty_set;;
module Mapping = Umatrix_graph;;
module Structural_mapping = Umatrix_structural_mapping;;

open Refinement_types;;

let same = Structural_mapping.same
and last_zone_number = Structural_mapping.last_block_number
;;

(* Partitions. *)

let partition_unique_domains pp0 cn pp =
  if cn = Nat._0_ then pp0 else Uarray.map2 Mapping.unique_domain pp0 pp
;;

let partition_to_geometry_and_values =
  Structural_mapping.partition_to_geometry_and_values
(*and partition_of_geometry_and_values =
  Structural_mapping.partition_of_geometry_and_values*)
and one_zone_partition = Structural_mapping.one_zone_partition
;;

(* Converters. *)

let partition_of_gradient = Structural_mapping.partition_of_mapping
and to_parameter = Structural_mapping.to_mapping
and of_partition = Structural_mapping.of_partition
and to_partition = Structural_mapping.to_partition
;;

(* Iterators. *)

let iter = Structural_mapping.iter
and fold_left = Structural_mapping.fold_left
and partition_iter = Structural_mapping.partition_iter
and iter_to_do = Structural_mapping.iter_to_do
and iter_value = Structural_mapping.iter_value
and fold_left_value = Structural_mapping.fold_left_value
;;

let force = Structural_mapping.to_array;;

(* Cuttings. *)

let cartesian_cuts ijmin ijmax dom p nct =
  (* nij = size of the box *)
  let nij = Nat.succ (Nat.sub ijmax ijmin) in
  (* nct is the maximum number of cuts,
     therefore nct + 1 is the maximum number of intervals.
     The value -1 means all possible cuts. *)
  let nint = if nct = Nat._0_ then nij else Nat.succ nct in
  let ijl = Nat.psplit nij nint in
  let rec loop accu = function
    | ([], _) -> assert false
    | (k :: kl, ijc) when ijc = ijmax ->
        if kl = [] then accu else
        loop accu (kl, Nat.sub ijc k)
    | (k :: kl, ijc) ->
        let dicho = Set.dichotomy (p ijc) dom in
        let accu = dicho :: accu in
        if kl = [] then accu else
        loop accu (kl, Nat.sub ijc k) in
  loop [] (ijl, ijmax)
;;

let horizontal_cuts { ll = (imin, _); ur = (imax, _); } dom nct =
  let pi ic idx _uidx =
    let (i, _) = Set.element dom idx in
    i <= ic in
  cartesian_cuts imin imax dom pi nct
;;

let vertical_cuts { ll = (_, jmin); ur = (_, jmax); } dom nct =
  let pj jc idx _uidx =
    let (_, j) = Set.element dom idx in
    j <= jc in
  cartesian_cuts jmin jmax dom pj nct
;;

let dichotomies_from_family bds dom nct = function
  | Horizontal_cut -> horizontal_cuts bds dom nct
  | Vertical_cut -> vertical_cuts bds dom nct
;;
